import abc

from poem_plugins.general.versions import Version


class IVervsionDriver(abc.ABC):
    @abc.abstractmethod
    def get_version(self, git_version_prefix: str = "v") -> Version:
        raise NotImplementedError

    @abc.abstractmethod
    def render_version_file(self, version: Version) -> str:
        raise NotImplementedError



from dataclasses import dataclass
from typing import Optional

from poem_plugins.general.strenum import StrEnum


class VersionEnum(StrEnum):
    GIT_LONG = "git-long"


@dataclass
class Config:
    version_plugin: Optional[VersionEnum] = None
    git_version_prefix: str = "v"
    update_pyproject: bool = False
    write_version_file: bool = False



from dataclasses import dataclass

from cleo.events.console_command_event import ConsoleCommandEvent
from cleo.events.event import Event
from cleo.events.event_dispatcher import EventDispatcher
from poetry.console.commands.build import BuildCommand
from poetry.core.utils.helpers import module_name
from poetry.poetry import Poetry
from tomlkit.toml_document import TOMLDocument

from poem_plugins.config import Config, VersionEnum
from poem_plugins.general.versions import Version
from poem_plugins.general.versions.drivers import IVervsionDriver
from poem_plugins.general.versions.drivers.git import GitLongVersionDriver


@dataclass(frozen=True)
class VersionDispatcher:
    config: Config
    driver: IVervsionDriver

    @classmethod
    def factory(cls, config: Config) -> "VersionDispatcher":
        if config.version_plugin == VersionEnum.GIT_LONG:
            driver = GitLongVersionDriver()
        else:
            driver = GitLongVersionDriver()
        return cls(
            driver=driver,
            config=config,
        )

    def _write_pyproject(
        self, poetry: Poetry, version: Version,
    ) -> None:
        if not self.config.update_pyproject:
            return
        content: TOMLDocument = poetry.file.read()
        poetry_content = content["tool"]["poetry"]  # type: ignore
        poetry_content["version"] = str(version)  # type: ignore
        poetry.file.write(content)

    def _write_module(
        self, poetry: Poetry, version: Version,
    ) -> None:
        if not self.config.write_version_file:
            return
        package_name = module_name(poetry.package.name)
        with open(f"{package_name}/version.py", "w") as file:
            file.write(
                self.driver.render_version_file(version=version),
            )

    def __call__(
        self, event: Event, event_name: str, dispatcher: EventDispatcher,
    ) -> None:
        if not isinstance(event, ConsoleCommandEvent):
            return
        command = event.command
        if not isinstance(command, BuildCommand):
            return
        if not self.config.version_plugin:
            return
        io = event.io
        poetry = command.poetry
        try:
            version = self.driver.get_version(self.config.git_version_prefix)
        except Exception as exc:
            io.write_error_line(f"<b>poem-plugins</b>: {exc}")
            raise exc

        io.write_line(
            f"<b>poem-plugins</b>: Setting version to: {version}",
        )
        poetry.package.version = str(version)  # type: ignore

        self._write_pyproject(poetry, version)
        self._write_module(poetry, version)



import re
import subprocess
import warnings
from shutil import which
from types import MappingProxyType
from typing import Any, Callable, ClassVar, Mapping, Match, Optional

from poem_plugins.general.versions import Version
from poem_plugins.general.versions.drivers import IVervsionDriver


GIT_BIN = which("git")
WARNING_TEXT = (
    '"git" binary was not found, this plugin this will not work properly'
)
if GIT_BIN is None:
    warnings.warn(WARNING_TEXT)


class GitLongVersionDriver(IVervsionDriver):
    VERSION_TEMPLATE = (
        "# THIS FILE WAS GENERATED BY \"{whoami}\"\n"
        "# NEWER EDIT THIS FILE MANUALLY\n"
        "\n"
        "version_info = ({major}, {minor}, {patch})\n"
        "__version__ = \"{version}\"\n"
    )

    CONVERTERS: ClassVar[Mapping[str, Callable[[Any], Any]]] = (
        MappingProxyType({
            "major": int,
            "minor": int,
            "patch": int,
        })
    )

    def get_version(self, git_version_prefix: str = "v") -> Version:
        if GIT_BIN is None:
            raise RuntimeError(WARNING_TEXT)

        result = subprocess.run(
            [GIT_BIN, "describe", "--long"],
            stdout=subprocess.PIPE,
            text=True,
        )

        if result.returncode != 0:
            raise RuntimeError("Cannot found git version")
        raw_version = result.stdout.strip()
        regex = (
            r"(?P<major>\d+)\.(?P<minor>\d+)-(?P<patch>\d+)-(?P<commit>\S+)"
        )
        match: Optional[Match[str]] = re.match(
            git_version_prefix + regex,
            raw_version,
        )

        if match is None:
            raise RuntimeError("Cannot parse git version")

        kwargs = {
            k: self.CONVERTERS.get(k, lambda x: x)(v)
            for k, v in match.groupdict().items()
        }

        return Version(**kwargs)

    def render_version_file(self, version: Version) -> str:
        return (
            self.VERSION_TEMPLATE.format(
                whoami=".".join((
                    self.__class__.__module__,
                    self.__class__.__name__,
                )),
                major=version.major,
                minor=version.minor,
                patch=version.patch,
                version=str(version),
            )
        )



from cleo.events.console_events import COMMAND
from poetry.console.application import Application

from poem_plugins.base import BasePlugin
from poem_plugins.versions.dispatcher import VersionDispatcher


class VersionsPlugin(BasePlugin):
    def activate(self, application: Application) -> None:
        if not application.event_dispatcher:
            return

        config = self.get_config(application.poetry)

        version_dispatcher = VersionDispatcher.factory(
            config=config,
        )
        application.event_dispatcher.add_listener(
            COMMAND, version_dispatcher,
        )



from typing import NamedTuple, Optional


class Version(NamedTuple):
    major: int
    minor: int
    patch: int
    commit: Optional[str] = None

    def __str__(self) -> str:
        if not self.commit:
            return f"{self.major}.{self.minor}.{self.patch}"
        return f"{self.major}.{self.minor}.{self.patch}+g{self.commit}"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['poem_plugins',
 'poem_plugins.dispatchers',
 'poem_plugins.general',
 'poem_plugins.general.versions',
 'poem_plugins.general.versions.drivers',
 'poem_plugins.plugins']

package_data = \
{'': ['*']}

install_requires = \
['poetry>=1.2,<2.0']

entry_points = \
{'poetry.application.plugin': ['poem-version-plugin = '
                               'poem_plugins.plugins:VersionsPlugin']}

setup_kwargs = {
    'name': 'poem-plugins',
    'version': '0.12.9',
    'description': 'Some set of poetry plugins',
    'long_description': '# Poem Plugins\n\n<a href="https://pypi.org/project/poem-plugins" target="_blank">\n    <img src="https://img.shields.io/pypi/v/poem-plugins?color=%2334D058&label=pypi%20package" alt="Package version">\n</a>\n\nA set of plugins for [**Poetry**](https://python-poetry.org/).\n\n\n## How to use\nMake sure you have Poetry version `1.2.0` or above. Read below for instructions to install it if you haven\'t.\n\n### Install Poem Plugins\n\nInstall this plugin to your Poetry:\n\n```console\n$ poetry self add poem-plugins\n```\n\nAdd tool section in project pyproject.toml\n\n```toml\n[tool.poem-plugins]\nversion_plugin = "git-long"\n# Version tags must be starts with this prefix\ngit_version_prefix = "v"\n```\n\nCreate a git tag, for example:\n\n```console\n$ git tag v0.1\n```\n\nNext, build your project. It will show an output like:\n\n```console\n$ poetry build\npoem-plugins: Setting version to: 0.1.0+g5ee9240\nBuilding awesome_package (0.1.0+g5ee9240)\n  - Building sdist\n  - Built awesome_package-0.1.0+g5ee9240.tar.gz\n  - Building wheel\n  - Built awesome_package-0.1.0+g5ee9240-py3-none-any.whl```\n```\n',
    'author': 'Ivan Sitkin',
    'author_email': 'alvinera@yandex.ru',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/alviner/poem-plugins',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from poetry.plugins.application_plugin import ApplicationPlugin
from poetry.poetry import Poetry

from poem_plugins.config import Config


class BasePlugin(ApplicationPlugin):
    def get_config(self, poetry: Poetry) -> Config:
        raw = poetry.pyproject.data.get("tool", {}).get("poem-plugins", {})
        return Config(**raw)
